@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2008, 2013 Red Hat, Inc.
+# Copyright (C) 2006-2008, 2013-2014 Red Hat, Inc.
 # Copyright (C) 2006 Daniel P. Berrange <berrange@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -20,22 +20,19 @@
 
 import logging
 
-# pylint: disable=E0611
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
-# pylint: enable=E0611
+
+import libvirt
 
 from virtinst import util
 
-from virtManager import vmmenu
-from virtManager import uiutil
-from virtManager.connection import vmmConnection
-from virtManager.baseclass import vmmGObjectUI
-from virtManager.graphwidgets import CellRendererSparkline
-
-import libvirt
+from . import vmmenu
+from . import uiutil
+from .baseclass import vmmGObjectUI
+from .graphwidgets import CellRendererSparkline
 
 # Number of data points for performance graphs
 GRAPH_LEN = 40
@@ -110,21 +107,19 @@ class vmmManager(vmmGObjectUI):
         "manager-closed": (GObject.SignalFlags.RUN_FIRST, None, []),
         "manager-opened": (GObject.SignalFlags.RUN_FIRST, None, []),
         "remove-conn": (GObject.SignalFlags.RUN_FIRST, None, [str]),
-        "add-default-conn": (GObject.SignalFlags.RUN_FIRST, None, []),
     }
 
     def __init__(self):
         vmmGObjectUI.__init__(self, "manager.ui", "vmm-manager")
 
-        self.ignore_pause = False
-
-        # Mapping of VM UUID -> tree model rows to
+        # Mapping of rowkey -> tree model rows to
         # allow O(1) access instead of O(n)
         self.rows = {}
 
         w, h = self.config.get_manager_window_size()
         self.topwin.set_default_size(w or 550, h or 550)
         self.prev_position = None
+        self._window_size = None
 
         self.vmmenu = vmmenu.VMActionMenu(self, self.current_vm)
         self.connmenu = Gtk.Menu()
@@ -194,14 +189,6 @@ class vmmManager(vmmGObjectUI):
         self.enable_polling(COL_NETWORK)
         self.enable_polling(COL_MEM)
 
-        # Select first list entry
-        vmlist = self.widget("vm-list")
-        if len(vmlist.get_model()) != 0:
-            vmlist.get_selection().select_iter(
-                vmlist.get_model().get_iter_first())
-
-        # Queue up the default connection detector
-        self.idle_emit("add-default-conn")
 
     ##################
     # Common methods #
@@ -241,12 +228,16 @@ class vmmManager(vmmGObjectUI):
         self.hostcpucol = None
         self.netcol = None
 
-        self.vmmenu.destroy()  # pylint: disable=E1101
+        self.vmmenu.destroy()
         self.vmmenu = None
         self.connmenu.destroy()
         self.connmenu = None
         self.connmenu_items = None
 
+        if self._window_size:
+            self.config.set_manager_window_size(*self._window_size)
+
+
     def is_visible(self):
         return bool(self.topwin.get_visible())
 
@@ -259,34 +250,34 @@ class vmmManager(vmmGObjectUI):
     ################
 
     def init_stats(self):
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_guest_cpu_usage_visible_changed(
                                 self.toggle_guest_cpu_usage_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_host_cpu_usage_visible_changed(
                                 self.toggle_host_cpu_usage_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_memory_usage_visible_changed(
                                 self.toggle_memory_usage_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_disk_io_visible_changed(
                                 self.toggle_disk_io_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_network_traffic_visible_changed(
                                 self.toggle_network_traffic_visible_widget))
 
         # Register callbacks with the global stats enable/disable values
         # that disable the associated vmlist widgets if reporting is disabled
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_cpu_poll_changed(
                 self.enable_polling, COL_GUEST_CPU))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_disk_poll_changed(
                 self.enable_polling, COL_DISK))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_net_poll_changed(
                 self.enable_polling, COL_NETWORK))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_memory_poll_changed(
                 self.enable_polling, COL_MEM))
 
@@ -427,7 +418,7 @@ class vmmManager(vmmGObjectUI):
     ##################
 
     def current_row(self):
-        return uiutil.get_list_selection(self.widget("vm-list"), None)
+        return uiutil.get_list_selected_row(self.widget("vm-list"))
 
     def current_vm(self):
         row = self.current_row()
@@ -447,12 +438,6 @@ class vmmManager(vmmGObjectUI):
         else:
             return handle.conn
 
-    def current_vmuuid(self):
-        vm = self.current_vm()
-        if vm is None:
-            return None
-        return vm.get_uuid()
-
     def current_conn_uri(self, default_selection=False):
         vmlist = self.widget("vm-list")
         model = vmlist.get_model()
@@ -474,11 +459,9 @@ class vmmManager(vmmGObjectUI):
     ####################
 
     def window_resized(self, ignore, event):
-        # Sometimes dimensions change when window isn't visible
         if not self.is_visible():
             return
-
-        self.config.set_manager_window_size(event.width, event.height)
+        self._window_size = (event.width, event.height)
 
     def exit_app(self, src_ignore=None, src2_ignore=None):
         self.emit("action-exit-app")
@@ -506,7 +489,7 @@ class vmmManager(vmmGObjectUI):
             return
 
         if vm:
-            self.emit("action-show-domain", conn.get_uri(), vm.get_uuid())
+            self.emit("action-show-domain", conn.get_uri(), vm.get_connkey())
         else:
             if not self.open_conn():
                 self.emit("action-show-host", conn.get_uri())
@@ -517,7 +500,7 @@ class vmmManager(vmmGObjectUI):
         if vm is None:
             self._do_delete_conn(conn)
         else:
-            self.emit("action-delete-domain", conn.get_uri(), vm.get_uuid())
+            self.emit("action-delete-domain", conn.get_uri(), vm.get_connkey())
 
     def _do_delete_conn(self, conn):
         if conn is None:
@@ -533,15 +516,12 @@ class vmmManager(vmmGObjectUI):
     def set_pause_state(self, state):
         src = self.widget("vm-pause")
         try:
-            self.ignore_pause = True
+            src.handler_block_by_func(self.pause_vm_button)
             src.set_active(state)
         finally:
-            self.ignore_pause = False
+            src.handler_unblock_by_func(self.pause_vm_button)
 
     def pause_vm_button(self, src):
-        if self.ignore_pause:
-            return
-
         do_pause = src.get_active()
 
         # Set button state back to original value: just let the status
@@ -555,36 +535,37 @@ class vmmManager(vmmGObjectUI):
 
     def start_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-run-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-run-domain", vm.conn.get_uri(), vm.get_connkey())
 
     def poweroff_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-shutdown-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-shutdown-domain",
+            vm.conn.get_uri(), vm.get_connkey())
 
     def pause_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-suspend-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-suspend-domain", vm.conn.get_uri(), vm.get_connkey())
 
     def resume_vm(self, ignore):
         vm = self.current_vm()
-        if vm is not None:
-            self.emit("action-resume-domain",
-                      vm.conn.get_uri(), vm.get_uuid())
+        if vm is None:
+            return
+        self.emit("action-resume-domain", vm.conn.get_uri(), vm.get_connkey())
 
     def close_conn(self, ignore):
         conn = self.current_conn()
-        if conn.get_state() != vmmConnection.STATE_DISCONNECTED:
+        if not conn.is_disconnected():
             conn.close()
 
     def open_conn(self, ignore=None):
         conn = self.current_conn()
-        if conn.get_state() == vmmConnection.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             conn.open()
             return True
 
@@ -596,45 +577,53 @@ class vmmManager(vmmGObjectUI):
     def vm_row_key(self, vm):
         return vm.get_uuid() + ":" + vm.conn.get_uri()
 
-    def vm_added(self, conn, vmuuid):
-        vm = conn.get_vm(vmuuid)
-        if self.vm_row_key(vm) in self.rows:
+    def vm_added(self, conn, connkey):
+        vm = conn.get_vm(connkey)
+        if not vm:
             return
 
-        vm.connect("config-changed", self.vm_config_changed)
-        vm.connect("status-changed", self.vm_status_changed)
+        row_key = self.vm_row_key(vm)
+        if row_key in self.rows:
+            return
+
+        row = self._build_row(None, vm)
+        parent = self.rows[conn.get_uri()].iter
+        model = self.widget("vm-list").get_model()
+        _iter = model.append(parent, row)
+        path = model.get_path(_iter)
+        self.rows[row_key] = model[path]
+
+        vm.connect("state-changed", self.vm_changed)
         vm.connect("resources-sampled", self.vm_row_updated)
         vm.connect("inspection-changed", self.vm_inspection_changed)
 
-        vmlist = self.widget("vm-list")
-        model = vmlist.get_model()
+        # Expand a connection when adding a vm to it
+        self.widget("vm-list").expand_row(model.get_path(parent), False)
 
-        self._append_vm(model, vm, conn)
-
-    def vm_removed(self, conn, vmuuid):
+    def vm_removed(self, conn, connkey):
         vmlist = self.widget("vm-list")
         model = vmlist.get_model()
 
         parent = self.rows[conn.get_uri()].iter
         for row in range(model.iter_n_children(parent)):
             vm = model[model.iter_nth_child(parent, row)][ROW_HANDLE]
-            if vm.get_uuid() == vmuuid:
+            if vm.get_connkey() == connkey:
                 model.remove(model.iter_nth_child(parent, row))
                 del self.rows[self.vm_row_key(vm)]
                 break
 
     def _build_conn_hint(self, conn):
         hint = conn.get_uri()
-        if conn.state == conn.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             hint += " (%s)" % _("Double click to connect")
         return hint
 
     def _build_conn_markup(self, conn, name):
         name = util.xml_escape(name)
         text = name
-        if conn.state == conn.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             text += " - " + _("Not Connected")
-        elif conn.state == conn.STATE_CONNECTING:
+        elif conn.is_connecting():
             text += " - " + _("Connecting...")
 
         markup = "<span size='smaller'>%s</span>" % text
@@ -642,7 +631,7 @@ class vmmManager(vmmGObjectUI):
 
     def _build_conn_color(self, conn):
         color = "#000000"
-        if conn.state == conn.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             color = "#5b5b5b"
         return color
 
@@ -654,7 +643,7 @@ class vmmManager(vmmGObjectUI):
 
     def _build_row(self, conn, vm):
         if conn:
-            name = conn.get_pretty_desc_inactive(False)
+            name = conn.get_pretty_desc()
             markup = self._build_conn_markup(conn, name)
             status = ("<span size='smaller'>%s</span>" %
                       conn.get_state_text())
@@ -679,7 +668,7 @@ class vmmManager(vmmGObjectUI):
         row.insert(ROW_HINT, util.xml_escape(hint))
         row.insert(ROW_IS_CONN, bool(conn))
         row.insert(ROW_IS_CONN_CONNECTED,
-                   bool(conn) and conn.state != conn.STATE_DISCONNECTED)
+                   bool(conn) and not conn.is_disconnected())
         row.insert(ROW_IS_VM, bool(vm))
         row.insert(ROW_IS_VM_RUNNING, bool(vm) and vm.is_active())
         row.insert(ROW_COLOR, color)
@@ -687,65 +676,29 @@ class vmmManager(vmmGObjectUI):
 
         return row
 
-    def _append_vm(self, model, vm, conn):
-        row_key = self.vm_row_key(vm)
-        if row_key in self.rows:
-            return
-
-        row = self._build_row(None, vm)
-        parent = self.rows[conn.get_uri()].iter
-
-        _iter = model.append(parent, row)
-        path = model.get_path(_iter)
-        self.rows[row_key] = model[path]
-
-        # Expand a connection when adding a vm to it
-        self.widget("vm-list").expand_row(model.get_path(parent), False)
-
-    def _append_conn(self, model, conn):
-        row = self._build_row(conn, None)
-
-        _iter = model.append(None, row)
-        path = model.get_path(_iter)
-        self.rows[conn.get_uri()] = model[path]
-        return _iter
-
     def add_conn(self, engine_ignore, conn):
+        # Called from engine.py signal conn-added
+
         # Make sure error page isn't showing
         self.widget("vm-notebook").set_current_page(0)
 
         if conn.get_uri() in self.rows:
             return
 
+        model = self.widget("vm-list").get_model()
+        row = self._build_row(conn, None)
+        _iter = model.append(None, row)
+        path = model.get_path(_iter)
+        self.rows[conn.get_uri()] = model[path]
+
         conn.connect("vm-added", self.vm_added)
         conn.connect("vm-removed", self.vm_removed)
         conn.connect("resources-sampled", self.conn_row_updated)
         conn.connect("state-changed", self.conn_state_changed)
 
-        # add the connection to the treeModel
-        vmlist = self.widget("vm-list")
-        row = self._append_conn(vmlist.get_model(), conn)
-        vmlist.get_selection().select_iter(row)
-
-        # Try to make sure that 2 row descriptions don't collide
-        connrows = []
-        descs = []
-        for row in self.rows.values():
-            if row[ROW_IS_CONN]:
-                connrows.append(row)
-        for row in connrows:
-            descs.append(row[ROW_SORT_KEY])
-
-        for row in connrows:
-            conn = row[ROW_HANDLE]
-            name = row[ROW_SORT_KEY]
-            if descs.count(name) <= 1:
-                continue
-
-            newname = conn.get_pretty_desc_inactive(False, True)
-            self.conn_state_changed(conn, newname=newname)
-
     def remove_conn(self, engine_ignore, uri):
+        # Called from engine.py signal conn-removed
+
         model = self.widget("vm-list").get_model()
         parent = self.rows[uri].iter
 
@@ -772,12 +725,15 @@ class vmmManager(vmmGObjectUI):
             return
         self.widget("vm-list").get_model().row_changed(row.path, row.iter)
 
-    def vm_config_changed(self, vm):
+    def vm_changed(self, vm):
         row = self.rows.get(self.vm_row_key(vm), None)
         if row is None:
             return
 
         try:
+            if vm == self.current_vm():
+                self.update_current_selection()
+
             name = vm.get_name_or_title()
             status = vm.run_status()
 
@@ -787,8 +743,6 @@ class vmmManager(vmmGObjectUI):
             row[ROW_MARKUP] = self._build_vm_markup(name, status)
 
             desc = vm.get_description()
-            if not uiutil.can_set_row_none:
-                desc = desc or ""
             row[ROW_HINT] = util.xml_escape(desc)
         except libvirt.libvirtError, e:
             if util.exception_is_libvirt_error(e, "VIR_ERR_NO_DOMAIN"):
@@ -797,50 +751,48 @@ class vmmManager(vmmGObjectUI):
 
         self.vm_row_updated(vm)
 
-    def vm_status_changed(self, vm, oldstatus, newstatus):
-        ignore = newstatus
-        ignore = oldstatus
-        parent = self.rows[vm.conn.get_uri()].iter
-        vmlist = self.widget("vm-list")
-        model = vmlist.get_model()
-
-        missing = True
-        for row in range(model.iter_n_children(parent)):
-            _iter = model.iter_nth_child(parent, row)
-            if model[_iter][ROW_HANDLE] == vm:
-                missing = False
-                break
-
-        if missing:
-            self._append_vm(model, vm, vm.conn)
-
-        # Update run/shutdown/pause button states
-        self.update_current_selection()
-        self.vm_config_changed(vm)
-
     def vm_inspection_changed(self, vm):
         row = self.rows.get(self.vm_row_key(vm), None)
         if row is None:
             return
 
         new_icon = _get_inspection_icon_pixbuf(vm, 16, 16)
-        if not uiutil.can_set_row_none:
-            new_icon = new_icon or ""
         row[ROW_INSPECTION_OS_ICON] = new_icon
 
         self.vm_row_updated(vm)
 
-    def conn_state_changed(self, conn, newname=None):
+    def set_initial_selection(self, uri):
+        vmlist = self.widget("vm-list")
+        model = vmlist.get_model()
+        it = model.get_iter_first()
+        selected = None
+        while it:
+            key = model.get_value(it, ROW_HANDLE)
+
+            if key.get_uri() == uri:
+                vmlist.get_selection().select_iter(it)
+                return
+
+            if not selected:
+                vmlist.get_selection().select_iter(it)
+                selected = key
+            elif key.get_autoconnect() and not selected.get_autoconnect():
+                vmlist.get_selection().select_iter(it)
+                selected = key
+                if not uri:
+                    return
+
+            it = model.iter_next(it)
+
+    def conn_state_changed(self, conn):
         row = self.rows[conn.get_uri()]
-        if newname:
-            row[ROW_SORT_KEY] = newname
+        row[ROW_SORT_KEY] = conn.get_pretty_desc()
         row[ROW_MARKUP] = self._build_conn_markup(conn, row[ROW_SORT_KEY])
-        row[ROW_IS_CONN_CONNECTED] = conn.state != conn.STATE_DISCONNECTED
+        row[ROW_IS_CONN_CONNECTED] = not conn.is_disconnected()
         row[ROW_COLOR] = self._build_conn_color(conn)
         row[ROW_HINT] = self._build_conn_hint(conn)
 
-        if conn.get_state() in [vmmConnection.STATE_DISCONNECTED,
-                                vmmConnection.STATE_CONNECTING]:
+        if not conn.is_active():
             # Connection went inactive, delete any VM child nodes
             parent = row.iter
             if parent is not None:
@@ -890,7 +842,7 @@ class vmmManager(vmmGObjectUI):
         show_shutdown = bool(vm and vm.is_stoppable())
 
         if vm and vm.managedsave_supported:
-            self.change_run_text(vm.hasSavedImage())
+            self.change_run_text(vm.has_managed_save())
 
         self.widget("vm-open").set_sensitive(show_open)
         self.widget("vm-run").set_sensitive(show_run)
@@ -900,6 +852,12 @@ class vmmManager(vmmGObjectUI):
         self.set_pause_state(is_paused)
         self.widget("vm-pause").set_sensitive(show_pause)
 
+        if is_paused:
+            pauseTooltip = _("Resume the virtual machine")
+        else:
+            pauseTooltip = _("Pause the virtual machine")
+        self.widget("vm-pause").set_tooltip_text(pauseTooltip)
+
         self.widget("menu_edit_details").set_sensitive(show_details)
         self.widget("menu_host_details").set_sensitive(host_details)
 
@@ -930,13 +888,12 @@ class vmmManager(vmmGObjectUI):
             # Popup the vm menu
             vm = model[_iter][ROW_HANDLE]
             self.vmmenu.update_widget_states(vm)
-            self.vmmenu.popup(  # pylint: disable=E1101
-                None, None, None, None, 0, event.time)
+            self.vmmenu.popup(None, None, None, None, 0, event.time)
         else:
             # Pop up connection menu
             conn = model[_iter][ROW_HANDLE]
-            disconn = (conn.get_state() == vmmConnection.STATE_DISCONNECTED)
-            conning = (conn.get_state() == vmmConnection.STATE_CONNECTING)
+            disconn = conn.is_disconnected()
+            conning = conn.is_connecting()
 
             self.connmenu_items["create"].set_sensitive(not disconn)
             self.connmenu_items["disconnect"].set_sensitive(not (disconn or
@@ -952,7 +909,9 @@ class vmmManager(vmmGObjectUI):
     #################
 
     def vmlist_name_sorter(self, model, iter1, iter2, ignore):
-        return cmp(model[iter1][ROW_SORT_KEY], model[iter2][ROW_SORT_KEY])
+        key1 = str(model[iter1][ROW_SORT_KEY]).lower()
+        key2 = str(model[iter2][ROW_SORT_KEY]).lower()
+        return cmp(key1, key2)
 
     def vmlist_guest_cpu_usage_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
@@ -988,6 +947,7 @@ class vmmManager(vmmGObjectUI):
         return cmp(obj1.network_traffic_rate(), obj2.network_traffic_rate())
 
     def enable_polling(self, column):
+        # pylint: disable=redefined-variable-type
         if column == COL_GUEST_CPU:
             widgn = ["menu_view_stats_guest_cpu", "menu_view_stats_host_cpu"]
             do_enable = self.config.get_stats_enable_cpu_poll()
@@ -1078,7 +1038,7 @@ class vmmManager(vmmGObjectUI):
         if obj is None or not hasattr(obj, "conn"):
             return
 
-        data = obj.guest_cpu_time_vector_limit(GRAPH_LEN)
+        data = obj.guest_cpu_time_vector(GRAPH_LEN)
         cell.set_property('data_array', data)
 
     def host_cpu_usage_img(self, column_ignore, cell, model, _iter, data):
@@ -1086,7 +1046,7 @@ class vmmManager(vmmGObjectUI):
         if obj is None or not hasattr(obj, "conn"):
             return
 
-        data = obj.host_cpu_time_vector_limit(GRAPH_LEN)
+        data = obj.host_cpu_time_vector(GRAPH_LEN)
         cell.set_property('data_array', data)
 
     def memory_usage_img(self, column_ignore, cell, model, _iter, data):
@@ -1094,7 +1054,7 @@ class vmmManager(vmmGObjectUI):
         if obj is None or not hasattr(obj, "conn"):
             return
 
-        data = obj.memory_usage_vector_limit(GRAPH_LEN)
+        data = obj.stats_memory_vector(GRAPH_LEN)
         cell.set_property('data_array', data)
 
     def disk_io_img(self, column_ignore, cell, model, _iter, data):
@@ -1102,7 +1062,8 @@ class vmmManager(vmmGObjectUI):
         if obj is None or not hasattr(obj, "conn"):
             return
 
-        data = obj.disk_io_vector_limit(GRAPH_LEN, self.max_disk_rate)
+        d1, d2 = obj.disk_io_vectors(GRAPH_LEN, self.max_disk_rate)
+        data = [(x + y) / 2 for x, y in zip(d1, d2)]
         cell.set_property('data_array', data)
 
     def network_traffic_img(self, column_ignore, cell, model, _iter, data):
@@ -1110,5 +1071,6 @@ class vmmManager(vmmGObjectUI):
         if obj is None or not hasattr(obj, "conn"):
             return
 
-        data = obj.network_traffic_vector_limit(GRAPH_LEN, self.max_net_rate)
+        d1, d2 = obj.network_traffic_vectors(GRAPH_LEN, self.max_net_rate)
+        data = [(x + y) / 2 for x, y in zip(d1, d2)]
         cell.set_property('data_array', data)
